/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * This interface is implemented by top level accessible object in hierarchy and
 * provides information about application.
 */
[scriptable, builtinclass, uuid(79251626-387c-4531-89f3-680d31d6cf05)]
interface nsIAccessibleApplication : nsISupports
{
  /**
   * Returns the application name.
   */
  readonly attribute DOMString appName;

  /**
   * Returns the application version.
   */
  readonly attribute DOMString appVersion;

  /**
   * Returns the platform name.
   */
  readonly attribute DOMString platformName;

  /**
   * Returns the platform version.
   */
  readonly attribute DOMString platformVersion;
};
